/*--------------------------------------------------------------------------*/
/** @file counter.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with photometric modules.

                  MASS project: TURBINA module file
                       header file for counter.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 2.0
*/
#ifndef COUNTER_H
#define COUNTER_H

#include <qarray.h>
#include "module.h"
#include "iocount.h"

#define BYTE_PER_COUNT	2	// in principle, may be "short" regime, then - one BYTE
#define NS2MS			(1.0E-6)	// nseconds to millisends conversion

#define MAX_LEVEL_VALUE			255
#define MIN_LEVEL_VALUE			0
#define MIN_EXPOSITION_NUMBER	1
#define MAX_EXPOSITION_NUMBER	32767
#define MIN_EXPOSE				0.9		// ms
#define MAX_EXPOSE				32.0	// ms
#define MIN_BLOCK_SIZE			1
#define MAX_BLOCK_SIZE			16

// Status of counter:

#define ACTIVE		0x01	// active_on
#define INDUCE		0x02	// induce_on
#define SHORT		0x04	// 1-byte data
#define SLAVE		0x08	// slave - 1, master - 0
#define SHUTTER		0x10	// shutter_on
#define TEST		0x20	// test
#define COUNT		0x80	// count

class BlockCounter {
private:
	int _number;
public:
	BlockCounter(void);
	int incr(void);
	int decr(void);
	int get(void) {return _number;}
	int set(int);
	void reset(void) {_number=0;}
};

class Counter : public HardModule {
private:
	const char*	_channel;
	uchar _stop;
	uchar _masterOn;
	uchar _masterOff;
	uchar _activeOn;
	uchar _activeOff;
	uchar _induceOn;
	uchar _induceOff;
	uchar _setInduc;
	uchar _getInduc;
	uchar _runMst;
	uchar _runTest;
	uchar _getNumber;
	uchar _setNumber;
	uchar _getBlSize;
	uchar _setBlSize;
	uchar _setLevel;
	uchar _getLevel;
	uchar _setExpos;
	uchar _getExpos;

	static const count_t INVALID_COUNT = max_count_t;
	int			_maxNData;
	count_t		*data;
	int			_nData;
	QArray<int>	blockLabel;
	int			_nBlockLabel;

	int			_test_errors;

	int			_levelValue;
	int			_exposValue;
	int			_expositionNumber;
	int			_blockSize;
	int			_lastLabelNumber;
	int			_lastLabel;

public:
	Counter(void);
	~Counter();

	void resetVariables(void);
	void create(int sizeOfData);
	void loadConst(void);

	void resizeData(int newSize);
	void resetData(void) {_nData=0; _nBlockLabel=0;}
	void resetTest(void) {_test_errors=-1;}
	void addBlockLabel(int newLabel);
	/**
	@return	The fact that data were expanded due to the loss */
	bool expandData(void);
	int repairData(void);
	int insertBlock(int label, count_t value);
	int removeBlock(int label);
	int putData(count_t dataVal);
	count_t* getData(void) {return(data);} 
	int getOutNumber(void) {return(_nData);}
	int setOutNumber(int number);
	int searchTestErrors(count_t test_number);


	char getModule(void) {return _module;}
	void setChannel(const char* channel) {_channel=channel;}
	const char* getChannel(void) {return _channel;}
	void setName(const char* name) {_identName=name;_identSection=name;}
	const char* getName(void) {return _identName;}

	int stop(int dialog=1);
	int setMasterOn(int dialog=1);
	int setMasterOff(int dialog=1);
	int setActiveOn(int dialog=1);
	int setActiveOff(int dialog=1);
	int setInduceOn(int dialog=1);
	int setInduceOff(int dialog=1);
	int setInduc(uchar masterAddress, int dialog=1);
	int setBlSize(uchar blockSize, int dialog=1);
	int setNumber(int expositionNumber, int dialog=1);
	int setLevel(double level, int dialog=1);
	double getLevel(void);
	int setExpos(double expos, int dialog=1);
	int runMst(int dialog=1);
	int runTest(int dialog=1);

	BlockCounter blockCounter;
};

#endif
